/*
  This file is under public domain.
*/

#include <windows.h>
#include <windowsx.h>
#include <commdlg.h>
#include <shlobj.h>
#include <tchar.h>

#include <stdlib.h>
#include <string.h>

#include "utils.h"
#include "select_directory.h"
#include "utils_lib.h"


static char *openfilename(char *name, int flags,
                        BOOL WINAPI (*GetFileName)(LPOPENFILENAME))
{
	OPENFILENAME ofn = {0};
	char lpFile[MAX_PATH];
	HWND hwnd;
	
	hwnd = get_browser_hwnd();
	if (hwnd == NULL) {
		return NULL;
	}
#if defined(WINVER) && _WIN32_WINNT >= 0x0500
	ofn.lStructSize = OPENFILENAME_SIZE_VERSION_400;
#else
	ofn.lStructSize = sizeof(ofn);
#endif
	lstrcpyn(lpFile, name, MAX_PATH);
	ofn.hwndOwner = hwnd;
	ofn.lpstrFilter = "All Files(*.*)\0*.*\0\0";
	ofn.nFilterIndex = 1;
	ofn.lpstrFile = lpFile;
	ofn.nMaxFile = MAX_PATH;
	ofn.Flags = flags;
	if ((*GetFileName)(&ofn)) {
		return _strdup(lpFile);
	}
	return NULL;
}


char *utils_GetOpenFileName(char *name)
{
	return openfilename(name,
		OFN_EXPLORER | OFN_FILEMUSTEXIST | OFN_HIDEREADONLY | OFN_NOCHANGEDIR,
		GetOpenFileName);
}


char *utils_GetSaveFileName(char *name)
{
	return openfilename(name,
		OFN_CREATEPROMPT | OFN_OVERWRITEPROMPT |
		OFN_EXPLORER | OFN_HIDEREADONLY | OFN_NOCHANGEDIR,
		GetSaveFileName);
}


char *utils_SelectDirectory(char *name)
{
	HWND hwnd;
	char lpFolder[MAX_PATH];
	
	hwnd = get_browser_hwnd();
	if (hwnd == NULL) {
		return NULL;
	}
	lstrcpyn(lpFolder, name, MAX_PATH);
	if (SelectDirectory(hwnd, "K-Meleon", NULL, lpFolder)) {
		return _strdup(lpFolder);
	}
	return NULL;
}


struct ListDialogParam {
	TCHAR *param;
	int answer;
};


static BOOL prepare_dialog(HWND hwnd, TCHAR *param)
{
	TCHAR *buffer = NULL, *p;
	size_t length = 0;
	HWND hwndDlgItem;
	
	if (param == NULL) return FALSE;
	p = next_line(&param, &buffer, &length);
	if (p == NULL || ! SetWindowText(hwnd, p)) {
		free(buffer);
		return FALSE;
	}
	if (param == NULL) return FALSE;
	p = next_line(&param, &buffer, &length);
	if (p == NULL || ! SetDlgItemText(hwnd, IDC_PROMPT, p)) {
		free(buffer);
		return FALSE;
	}
	if (param == NULL) return FALSE;
	hwndDlgItem = GetDlgItem(hwnd, IDC_LIST);
	do {
		p = next_line(&param, &buffer, &length);
		if (p == NULL ||
		    (_tcslen(p) > 0 && ListBox_AddString(hwndDlgItem, p) < 0)) {
			free(buffer);
			return FALSE;
		}
	} while (param != NULL);
	free(buffer);
	return TRUE;
}


static BOOL CALLBACK
ListDlgProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	static const TCHAR lpPropName[] = _T("param");
	struct ListDialogParam *param;
	HWND hwndDlgItem;
	
	switch (message) {
	case WM_INITDIALOG:
		param = (struct ListDialogParam *) lParam;
		if (! prepare_dialog(hwnd, param->param)) {
			EndDialog(hwnd, IDCANCEL);
			return TRUE;
		}
		SetProp(hwnd, lpPropName, param);
		return TRUE;
	case WM_COMMAND:
		if (LBN_DBLCLK == HIWORD(wParam) || IDOK == LOWORD(wParam)) {
			param = (struct ListDialogParam *) GetProp(hwnd, lpPropName);
			if (param) {
				hwndDlgItem = GetDlgItem(hwnd, IDC_LIST);
				param->answer = ListBox_GetCurSel(hwndDlgItem);
			}
			RemoveProp(hwnd, lpPropName);
			EndDialog(hwnd, IDOK);
		} else if (IDCANCEL == LOWORD(wParam)) {
			RemoveProp(hwnd, lpPropName);
			EndDialog(hwnd, IDCANCEL);
		}
		break;
	default:
		return FALSE;
	}
	return TRUE;
}


long utils_Listbox(char *param)
{
	HWND hwnd;
	struct ListDialogParam p;
	int ret;
	
	if ((hwnd = get_browser_hwnd()) == NULL) return -1;
	p.param = param;
	p.answer = LB_ERR;
	ret = DialogBoxParam(get_plugin_instance(),
	                     MAKEINTRESOURCE(IDD_LIST), hwnd,
	                     (DLGPROC) ListDlgProc, (LPARAM) &p);
	PostMessage(hwnd, WM_NULL, 0, 0);
	if (ret == IDOK && p.answer != LB_ERR) {
		return p.answer + 1;
		/* +1 is for compatibility with PopupMenu function. */
	}
	return -1;
}


long utils_PopupMenu(char *param)
{
	HMENU menu;
	HWND hwnd;
	TCHAR *buffer = NULL, *p;
	size_t length = 0;
	int i = 1;
	int r = -1;
	
	if (param == NULL) return r;
	if ((hwnd = get_browser_hwnd()) == NULL) return r;
	if ((menu = CreatePopupMenu()) == NULL) return r;
	do {
		p = next_line(&param, &buffer, &length);
		if (p == NULL) break;
		if (_tcslen(p) > 0) {
			if (! AppendMenu(menu, MF_ENABLED | MF_STRING, i ++, p)) break;
		} else {
			if (! AppendMenu(menu, MF_SEPARATOR, 0, NULL)) break;
		}
	} while (param != NULL);
	free(buffer);
	if (param == NULL && GetMenuItemCount(menu) > 0) {
		POINT pt = {0};
		GetCursorPos(&pt);
		r = TrackPopupMenu(menu, TPM_TOPALIGN | TPM_LEFTALIGN | TPM_RETURNCMD | TPM_NONOTIFY, pt.x, pt.y, 0, hwnd, NULL);
		PostMessage(hwnd, WM_NULL, 0, 0);
	}
	DestroyMenu(menu);
	return r;
}
