/*
  This file is under public domain.
*/
#include <windows.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <process.h>

#include "utils.h"

#define TIMEOUT_MAX (30 * 1000)
#define STRFTIME_MAX 256
#define COMMAND_LENGTH 256

char *utils_getenv(char *name)
{
	char *value = NULL;
	int size;
	
	size = GetEnvironmentVariable(name, NULL, 0);
	if (size > 0) {
		value = (char *) malloc(size);
		if (value) {
			GetEnvironmentVariable(name, value, size);
		}
	}
	return value;
}


void utils_setenv(char *name, char *value)
{
	SetEnvironmentVariable(name, value);
}


static HANDLE execute_command(char *cmd, STARTUPINFO *si)
{
	PROCESS_INFORMATION pi = {0};
	BOOL ret;
	DWORD length;
	char cmdline[COMMAND_LENGTH];
	
	length = ExpandEnvironmentStrings(cmd, cmdline, COMMAND_LENGTH);
	if (length > COMMAND_LENGTH) {
		char *p;
		p = (char *) malloc(sizeof(char) * length);
		if (! p) {
			return NULL;
		}
		ExpandEnvironmentStrings(cmd, p, length);
		cmd = p;
	} else {
		cmd = cmdline;
	}
	ret = CreateProcess(NULL, cmd, NULL, NULL, FALSE, NORMAL_PRIORITY_CLASS,
	                    NULL, NULL, si, &pi);
	if (cmd != cmdline) {
		free(cmd);
	}
	if (ret) {
		CloseHandle(pi.hThread);
		return pi.hProcess;
	}
	return NULL;
}


void utils_exec(char *cmd, char *wait)
{
	STARTUPINFO si = {0};
	PROCESS_INFORMATION pi = {0};
	HANDLE hProcess;
	int w = atoi(wait);
	
	si.cb = sizeof(STARTUPINFO);
	if (w < 0) {
		w = -w;
		si.dwFlags = STARTF_USESHOWWINDOW;
		si.wShowWindow = SW_SHOWMINNOACTIVE;
	}
	if (w > TIMEOUT_MAX) {
		w = TIMEOUT_MAX;
	}
	hProcess = execute_command(cmd, &si);
	if (hProcess) {
		if (w > 0) {
			WaitForSingleObject(pi.hProcess, w);
		}
		CloseHandle(hProcess);
	}
}


#define EXEC_SUCCESS 0
#define EXEC_CANCEL  1
#define EXEC_ERROR   2


struct exec_param {
	char *cmd;
	HWND hwndDialog;
	HANDLE hEvent;
	int result;
};


static HANDLE execute(char *cmd)
{
	STARTUPINFO si = {0};
	
	si.cb = sizeof(STARTUPINFO);
	return execute_command(cmd, &si);
}


static unsigned __stdcall exec_thread_proc(void *param)
{
	struct exec_param *p = (struct exec_param *) param;
	HANDLE hProcess;
	
	hProcess = execute(p->cmd);
	if (hProcess) {
		HANDLE h[2] = {hProcess, p->hEvent};
		
		switch (WaitForMultipleObjects(2, h, FALSE, INFINITE)) {
		case WAIT_OBJECT_0:
			p->result = EXEC_SUCCESS;
			break;
		case WAIT_OBJECT_0 + 1:
			p->result = EXEC_CANCEL;
			break;
		}
		CloseHandle(hProcess);
	}
	PostMessage(p->hwndDialog, WM_COMMAND, MAKEWPARAM(IDOK, 0), 0);
	_endthreadex(0);
	return 0;
}


static BOOL start_thread(struct exec_param *p)
{
	HANDLE hThread;
	unsigned tid;
	
	hThread = (HANDLE) _beginthreadex(NULL, 0, exec_thread_proc, p, 0, &tid);
	if (hThread) {
		CloseHandle((HANDLE) hThread);
		return TRUE;
	}
	return FALSE;
}


static BOOL CALLBACK ExecDialogProc(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	struct exec_param *p;
	
	switch (uMsg) {
	case WM_INITDIALOG:
		SetDlgItemText(hwnd, IDC_LABEL, translate(LABEL_EXECUTE));
		SetDlgItemText(hwnd, IDCANCEL, translate(LABEL_CANCEL));
		p = (struct exec_param *) lParam;
		SetWindowLongPtr(hwnd, DWL_USER, (LONG_PTR) p);
		p->hwndDialog = hwnd;
		if (! start_thread(p)) {
			EndDialog(hwnd, TRUE);
		}
		break;
	case WM_COMMAND:
		switch (LOWORD(wParam)) {
		case IDOK:
			EndDialog(hwnd, TRUE);
			break;
		case IDCANCEL:
			p = (struct exec_param *) GetWindowLongPtr(hwnd, DWL_USER);
			if (p) {
				SetEvent(p->hEvent);
			}
			break;
		}
		return TRUE;
	}
	return FALSE;
}


static HANDLE create_event(void)
{
	SECURITY_ATTRIBUTES sa;
	
	sa.nLength = sizeof(sa);
	sa.lpSecurityDescriptor = NULL;
	sa.bInheritHandle = FALSE;
	return CreateEvent(&sa, TRUE, FALSE, NULL);
}


char *utils_execex(char *cmd)
{
	HINSTANCE hInst = get_plugin_instance();
	HWND hwnd = get_browser_hwnd();
	struct exec_param p;
	
	p.cmd = cmd;
	p.hEvent = create_event();
	p.result = EXEC_ERROR;
	if (p.hEvent) {
		DialogBoxParam(hInst, MAKEINTRESOURCE(IDD_WAIT), hwnd,
			ExecDialogProc, (LPARAM) &p);
		CloseHandle(p.hEvent);
	}
	if (p.result == EXEC_SUCCESS) {
		return _strdup("true");
	}
	return _strdup("false");
}


void utils_sleep(char *wait, char *dummy)
{
	int w = atoi(wait);
	if (w < 0 || w > TIMEOUT_MAX) {
		w = TIMEOUT_MAX;
	}
	Sleep(w);
}


char *utils_getosver(char *field)
{
#define ARGCHECK(a, b) (a != NULL && strnicmp(a, b, sizeof(b) - 1) == 0)
#define ULSTRLEN 33

	char *value = NULL;
	OSVERSIONINFO osvi;
	
	osvi.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
	if (GetVersionEx(&osvi)) {
		if (ARGCHECK(field, "major")) {
			value = (char *) malloc(ULSTRLEN);
			if (value) {
				_itoa(osvi.dwMajorVersion, value, 10);
			}
		} else if (ARGCHECK(field, "minor")) {
			value = (char *) malloc(ULSTRLEN);
			if (value) {
				_itoa(osvi.dwMinorVersion, value, 10);
			}
		} else if (ARGCHECK(field, "build")) {
			value = (char *) malloc(ULSTRLEN);
			if (value) {
				if (osvi.dwPlatformId == VER_PLATFORM_WIN32_NT) {
					_itoa(osvi.dwBuildNumber, value, 10);
				} else {
					_itoa(LOWORD(osvi.dwBuildNumber), value, 10);
				}
			}
		} else if (ARGCHECK(field, "platform")) {
			value = (char *) malloc(ULSTRLEN);
			if (value) {
				_itoa(osvi.dwPlatformId, value, 10);
			}
		} else if (ARGCHECK(field, "csd")) {
			value = (char *) malloc(strlen(osvi.szCSDVersion) + 1);
			if (value) {
				strcpy(value, osvi.szCSDVersion);
			}
		} else {
			value = (char *) malloc(ULSTRLEN * 4 + 4 + strlen(osvi.szCSDVersion) + 1);
			if (value) {
				DWORD build;
				
				if (osvi.dwPlatformId == VER_PLATFORM_WIN32_NT) {
					build = osvi.dwBuildNumber;
				} else {
					build = LOWORD(osvi.dwBuildNumber);
				}
				sprintf(value, "%lu,%lu,%lu,%lu,%s",
				        osvi.dwMajorVersion, osvi.dwMinorVersion,
				        build, osvi.dwPlatformId, osvi.szCSDVersion);
			}
		}
	}
	return value;
#undef ULSTRLEN
#undef ARGCHECK
}

char *utils_strftime(char *format)
{
	time_t t;
	struct tm *tp;
	char *s;
	
	if (time(&t) < 0) return NULL;
	tp= localtime(&t);
	if (tp == NULL) return NULL;
	s = static_cast<char *>(malloc(STRFTIME_MAX));
	if (s == NULL) return NULL;
	if (strftime(s, STRFTIME_MAX, format, tp)) {
		return s;
	}
	free(s);
	return NULL;
}
